/************************************************************************
              WinControl Command Declarations (General)

              This is a part outof WinControl SDK package.
              Version History:
              v1.00 - 1999/09/03*
              v1.30 - 2000/01/26 - Major Update, first public release

              Last modifyed 2000/01/26
              (c)1999-2000 HALAN Innovations, all rights reserved
              Author: Daniel Halan (daniel@wincontrol.com)

**************************************************************************/

#ifndef KMD_HEADER
#define KMD_HEADER

#include <windows.h>

const HIGH_DWORD		= 4294967295;
const FRAME_OFFSET		= 80;
const STD_CTRL_OFFSET	= 40;
const STD_LABEL_OFFSET	= 16;


enum wError {ERR_NOPROBLEM,ERR_OUT_OF_MEMORY,ERR_NO_SUCH_OBJECT,ERR_PARAMETER,ERR_FREELIBRARY,ERR_UNKNOWN_ORDER,ERR_UNKNOWN};
enum wOSType {osWin16,osWinNT,osWin9X,osMac,osUnix,osAmiga};
enum wEventCall {ecInitize,ecDispose};
enum wOrder {oThis,oNext,oPrev,oFirst,oLast,oRand};


	struct wVisible 
	{
		long Item;		// what item in Array should be Selected for this control to show
		long ItemIndex; // what itemindex should be active to show this Control
	};


	// Only used if there is a problem with version information in Resource.
	struct wVer
	{
		WORD Major;
		WORD Minor;
		WORD Build;
		char *Extra; // additional information, ie. 'Beta'
	};

	// !!Important!!
	// Version Information is read from the Resource file, so be sure to update
	// it before release. You can do that thru "Project->Options->Version Info"
	struct wCmdInfo 
	{
    char *Name,		// Plugin name, aprears in the About Dialog
		 *Author,	// Your name
		 *Company,	// Your Company name
		 *VerInfo,  // Additional Version Info, ie Beta/Release
		 *Desc,     // Description;  #13 = LineFeed
		 *HelpFile; // Path & FileName. Only FileName = Kmd-Dir.
	wVer Ver;		// If there is problems with the Resouce Version this will be used
	DWORD Reserved[4]; // Unused
	};

	enum wParamType {ptEdit,ptIntEdit,ptCombo,ptCheckBox,
					 ptRadioGroup,ptBevel,ptPageControl,ptButton};
  
	
	struct wParamValue
	{
		wParamType vType;
		void *Data;			// Visual Data Type, ie. pEdit, pComboBox
		long int Value;		// Int/PChar
	};
	//#define wParamValue ;
  
	struct wParam
	{
    char	*Name;	// Plugin Name, !Important! Don't change the name after the
                    // first release, as it is a part of the Uniqe ID your plugin
                    // gets. If you change this, all saved Trigger with your Command will be deleted.
    DWORD	hIcon;	// Default Icon Handle      
    WORD	Height; // 0 will use Default value (Default = 200)
    WORD	Width;	// 0 will use Default value (Default = 360)
    WORD	ValueCount;// How many Parameters you will use
	void	**Values;//      : array of *wParamValue;
	};
	#define pParam *wParam;

  // This is the Edit-Control Values in SetupWindow
  // Requested when Updating a Trigger.
	struct wMainText 
	{
		char   *Text;		// The text that will apear in the SetupWindow
		HICON	Icon;		// the Icon that will apear beside the text in SetupWindow
		BOOL	ReadOnly;	// if the user can change the text or not. if TRUE the user won't
							// have access to use the keyboard in the EditBox.
	};
	#define pMainText *wMainText;

	typedef void  (__stdcall *CB_D_PChar) (char *);		
	typedef char* (__stdcall *CB_N_PChar) (char *, long); 

	struct wMainValues // Data from WinControl retrived at Main-Init
	{
		HWND		WndHandle;		// The SetupWindow Handle. use this for various API Calls
		wOSType		RunOS;			// The Running Operating System
		CB_N_PChar	NewPCharCB;		// Allocate New PChar
		CB_D_PChar	DisposePCharCB;	// Dispose PChar allocated with NewPChar()
	    DWORD		Reserved[4];	// Don't change/use
	};

	enum wButtonType {btNone,btFile,btDir,btOther};	// wEdit/wBtn
	enum wEditExtras {eNone,eFileBtn,eDirBtn};		// wEdit

  ////////////////////////////////////
  // Command Base Class - CBC

  // Visual Component Types
  typedef long (__stdcall *wOnChangeCB)  (long);// Callback that get's
												// called when a user
												// edits the Control.
												// the Returned value
												// will be used.
												// if you catch this you
												// MUST return an Int-value.
												// If you catch this from "wEdit"
												// you will recive an PChar
                                                // if you want to change it
                                                // you need to allocate a new
                                                // PChar with NewPChar(), and Free
                                                // the old Str with FreePChar().
                                                // then Return it as Integer
                                                // if you don't want to change the PChar, return -1.

	struct wVisObj
	{
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
	};

	struct wPageControl
	{
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		WORD		ItemCount;	// The number of Items
		long		ItemIndex;	// What item should be selected as default
		wOnChangeCB DoOnChange; // Callback function when user changes value
		char		**Items;
	};

	struct wRadioGroup 
	{
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		WORD		ItemCount;	// How many Items Exists
		long		ItemIndex;	// What item should be selected as default
		wOnChangeCB DoOnChange;	// Callback function when user changes value
		char		**Items;    //: array of PChar; (Items)
	};

	struct wComboBox  
	{ 
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		WORD		ItemCount;	// How many Items
		long		ItemIndex;	// What item should be selected as default
		wOnChangeCB DoOnChange;	// Callback function when user changes value
		char		**Items;//     : array of PChar;
	};

	struct wCheckBox
	{
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
 		// Item Specific
		BOOL Checked;			// if Checked as default
		wOnChangeCB DoOnChange;	// Callback function when user changes value
	};

	struct wEdit 
    {
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		char		*Value;		// Return Value
		wEditExtras	Extras;		// Extra Attrib.
		char		*ExtraExt;	// ie. FileBtn = '*.exe;*.bat'
		WORD		Max;		// Max Char Length
		wOnChangeCB DoOnChange;	// Callback function when user changes value
	};

	struct wIntEdit 
    {
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		long Value;				// The Value
		long Min;				// Minimun Value
		long Max;				// Maximum Value
		wOnChangeCB DoOnChange;	// Callback function when user changes value
	};

	struct wFloatEdit     
	{
		char		*Text;		// The Caption
		byte		PropWidth;	// 0-100 %
		byte		PropLeft;	// 0-100 %
		WORD		Height;		// pixels
		byte		TopPos;		// 1 = topmost, 2 = 2nd from top
		long		HelpContext;// HelpPage to show when user press F1 
		WORD		VisCount;	// How many VisObjects. if 0 this Objects is always visible
		wVisible	**Visible;	// Array of Visible Objects. when this objects should be visible
		// Item Specific
		double Value,Min,Max; 
		wOnChangeCB DoOnChange;	// Callback function when user changes value
	};


extern wMainValues wincon; // WinControl Information/Callbacks
	

wParamValue* CreateEdit(char *pcText,char *pcValue, wEditExtras eExtra, char *eExtraExt,
WORD iMax, long hContx, WORD VisCount, wVisible *pvArr[], byte iPropWidth, 
byte iPropLeft, byte iTopPos,  wOnChangeCB ocCB = NULL);

wParamValue* CreateIntEdit(char *pcText, long iValue, long iMax, long iMin, long hContx,
WORD VisCount, wVisible *pvArr[], byte iPropWidth, byte iPropLeft, 
byte iTopPos, wOnChangeCB ocCB = NULL);

wParamValue* CreateComboBox(char *pcText, WORD iCount, char *pcItems[], // array of char*
long iItemIndex, long hContx, WORD VisCount, wVisible *pvArr[], byte iPropWidth, 
byte iPropLeft, byte iTopPos, wOnChangeCB ocCB = NULL);

wParamValue *CreateRadioGroup(char *pcText, WORD iCount,char *pcItems[], //: array of char*
long iItemIndex, long hContx, WORD VisCount, wVisible *pvArr[], byte iPropWidth, byte iPropLeft, 
byte iTopPos, WORD iHeight, wOnChangeCB ocCB = NULL);

wParamValue* CreateCheckBox(char *pcText, BOOL bChecked, long hContx,
long VisCount, wVisible *pvVis[], byte iPropWidth, byte iPropLeft, byte iTopPos, wOnChangeCB ocCB = NULL);

wParam* NewParams(char *pcName, WORD iCount, WORD iHeight = 0, WORD iWidth = 0, HICON iIcon = 0);
wError FreeParams(wParam *Params);

inline void *NewArr(DWORD Count) { return realloc( NULL,(Count*sizeof(void*)) ); };
inline void DispArr(void *P) { free(P); };
inline char* NewPChar(char *Str, long Length = 0) { return wincon.NewPCharCB(Str,Length); };
inline void  FreePChar(char *Str) { wincon.DisposePCharCB(Str); };
inline void AssignWinCon(wMainValues *Main)
{ wincon.WndHandle = Main->WndHandle; wincon.RunOS = Main->RunOS; wincon.NewPCharCB = Main->NewPCharCB; 
  wincon.DisposePCharCB = Main->DisposePCharCB; };

inline void SetPrmValue(wParamValue *Prm, long iValue = 0, char *Str = NULL)
{ if (Str==NULL) Prm->Value = (DWORD)iValue; else Prm->Value = (DWORD)Str;};

inline long GetPrmValue(wParamValue *Prm)
{ if (Prm->vType != ptEdit) return ((((DWORD)Prm->Value)-(HIGH_DWORD))-1); else return 0;};

inline BOOL CmpPrmValue(wParamValue *Prm, long iValue)
{ return ((long)GetPrmValue(Prm)==iValue); };

inline void IncPrmValue(wParamValue *Prm)
{ if (Prm->vType != ptEdit) Prm->Value++; };

inline void DecPrmValue(wParamValue *Prm)
{ if (Prm->vType != ptEdit) Prm->Value--; };



#endif